#include <windows.h>
#include <wininet.h>
#include <stdio.h>

/// Checks if the haystack begins with the needle
bool StringBeginsWith(__in const wchar_t *lpszHaystack, __in const wchar_t *lpszNeedle)
{
	while (*lpszNeedle != L'\0' && *lpszHaystack != L'\0')
	{
		if (*lpszNeedle != *lpszHaystack)
		{
			return false;
		}

		lpszNeedle++;
		lpszHaystack++;
	}

	return *lpszNeedle == L'\0';
}

void UrlParse(__in const wchar_t *lpszUrl, __out wchar_t **ppHostName, __out wchar_t **ppPath)
{
	URL_COMPONENTS URLparts;

	URLparts.dwStructSize = sizeof( URLparts );

	// The following elements determine which components are displayed
	URLparts.dwSchemeLength    = 0;
	URLparts.dwHostNameLength  = 1;
	URLparts.dwUserNameLength  = 0;
	URLparts.dwPasswordLength  = 0;
	URLparts.dwUrlPathLength   = 1;
	URLparts.dwExtraInfoLength = 0;

	URLparts.lpszScheme     = NULL;
	URLparts.lpszHostName   = NULL;
	URLparts.lpszUserName   = NULL;
	URLparts.lpszPassword   = NULL;
	URLparts.lpszUrlPath    = NULL;
	URLparts.lpszExtraInfo  = NULL;

	DWORD n = wcsnlen_s(lpszUrl, 1024) + 8;
	wchar_t *UrlWithHttp = new wchar_t[n];

	// if the string does not begin with http://, insert it there.
	if (StringBeginsWith(lpszUrl, L"http://"))
	{
		swprintf_s(UrlWithHttp, n, L"%s", lpszUrl);
	}
	else
	{
		swprintf_s(UrlWithHttp, n, L"http://%s", lpszUrl);
	}

	// split the url into components and retrieve the hostname and path. Note that these are merely
	// pointers to the original string, so to return them, we have to create new strings and copy the characters
	if (InternetCrackUrl(UrlWithHttp, n, 0, &URLparts))
	{
		*ppHostName = new wchar_t[URLparts.dwHostNameLength + 1];
		memcpy(*ppHostName, URLparts.lpszHostName, URLparts.dwHostNameLength * sizeof(wchar_t));
		(*ppHostName)[URLparts.dwHostNameLength] = L'\0';

		*ppPath = new wchar_t[URLparts.dwUrlPathLength + 1];
		memcpy(*ppPath, URLparts.lpszUrlPath, URLparts.dwUrlPathLength * sizeof(wchar_t));
		(*ppPath)[URLparts.dwUrlPathLength] = L'\0';
	}

	delete[] UrlWithHttp;
}

void FindAndPrintHeaders(__in const wchar_t* hostname, __in const wchar_t* path)
{
	HINTERNET hOpenHandle,  hConnectHandle, hResourceHandle;
	DWORD dwError, dwErrorCode;
	HWND hwnd = GetConsoleWindow();

	hOpenHandle = InternetOpen(L"Example",
		INTERNET_OPEN_TYPE_PRECONFIG, 
		NULL, NULL, 0);

	hConnectHandle = InternetConnect(hOpenHandle,
		hostname, 
		INTERNET_INVALID_PORT_NUMBER,
		NULL,
		NULL, 
		INTERNET_SERVICE_HTTP,
		0,0);

	hResourceHandle = HttpOpenRequest(hConnectHandle, L"GET",
		path,
		NULL, NULL, NULL, 
		INTERNET_FLAG_NO_AUTO_REDIRECT, 0);

	HttpSendRequest(hResourceHandle, NULL, 0, NULL, 0);

	// dwErrorCode stores the error code associated with the call to
	// HttpSendRequest.  

	dwErrorCode = hResourceHandle ? ERROR_SUCCESS : GetLastError();

	dwError = InternetErrorDlg(hwnd, hResourceHandle, dwErrorCode, 
		FLAGS_ERROR_UI_FILTER_FOR_ERRORS | 
		FLAGS_ERROR_UI_FLAGS_CHANGE_OPTIONS |
		FLAGS_ERROR_UI_FLAGS_GENERATE_DATA,
		NULL);

	wchar_t *buf = NULL;//new wchar_t[4096];
	DWORD bufLength = 0;
	// this call will fail, but will give us the size of the buffer we need.
	HttpQueryInfo(hResourceHandle, HTTP_QUERY_RAW_HEADERS, buf, &bufLength, NULL);

	buf = new wchar_t[bufLength];

	if (HttpQueryInfo(hResourceHandle, HTTP_QUERY_RAW_HEADERS, buf, &bufLength, NULL))
	{
		//printf("Success:\r\n");
		wchar_t *current = buf;
		wchar_t *end = buf + bufLength;
		while(current < end && (*current != L'\0'))
		{
			int n = wcsnlen_s(current, end - current);
			wprintf(L"%s\r\n", current);
			current += (n+1);
		}
	}
	else
	{
		printf("Failed: \r\n");
	}

	delete[] buf;

	InternetCloseHandle(hResourceHandle);
	InternetCloseHandle(hConnectHandle);
	InternetCloseHandle(hOpenHandle);
}

void wmain(int argc, wchar_t** argv)
{
	if (argc < 2)
	{
		printf("httphead (C) 2011 Sushovan De, Arizona State University\r\n");
		printf("sends an HTTP HEAD request to a server of your choice and prints the response\r\n");
		printf("Usage: \r\n");
		printf("  httphead http://t.co/1234 \r\n");
		printf("  httphead t.co/1234 \r\n");
		return;
	}
	else
	{
		wchar_t *HostName;
		wchar_t *Path;
		UrlParse(argv[1], &HostName, &Path);
		//wprintf(L"Hostname: %s\r\n", HostName);
		//wprintf(L"Path:     %s\r\n\r\n", Path);
		FindAndPrintHeaders(HostName, Path);
	}
}
